@@ -11,6 +11,7 @@ summary.INLAjoint <- function(object, ...){
   if (!"INLAjoint" %in% class(object)){
     stop("Please provide an object of class 'INLAjoint' (obtained with joint() function).\n")
   }
+  if("run" %in% names(object)) if(!object$run) stop("Please run the model (with function `joint.run()`)")
   out <- NULL
   class(object) <- "inla"
   m.lstat.1 <- function(m) { #SD
@@ -39,7 +40,7 @@ summary.INLAjoint <- function(object, ...){
   Lmark <- Mark[grep("L", Mark)] # Longitudinal marker(s)
   Smark <- object$survOutcome #Mark[grep("S", Mark)] # Survival outcome(s)
   NLongi <- length(unique(Lmark))
-  NSurv <- length(unique(Smark))
+  NSurv <- length(Smark)
   Hnames <- rownames(object$summary.hyperpar)
   BH_temp <- object$summary.hyperpar[which(substring(Hnames, nchar(Hnames)-5, nchar(Hnames))=="hazard"), -which(colnames(object$summary.hyperpar)=="mode")]
   BH <- NULL
@@ -113,10 +114,43 @@ summary.INLAjoint <- function(object, ...){
   }
   REidentify <- which(substring(Hnames, 1, 5)=="Theta" | # extract random effects from hyperparameters
                         (substring(Hnames, 1, 16)=="Precision for ID"))
+  if(length(REidentify)==0 & !is.null(Hnames)){
+    REidentify <- intersect(which(substring(Hnames, 1, 14)=="Precision for "), # unusual name?
+                            sapply(unique(c(object[["REstruc"]], object[["REstrucS"]])), function(x) grep(x, Hnames)))
+  }
+  RandSpace <- NULL # identify spatial random effects (to avoid assuming they are frailty or iid)
+  if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_L", rownames(object$summary.hyperpar))))>0){ # spatial
+    SpaceEffi <- object$summary.hyperpar[intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_L", rownames(object$summary.hyperpar))), -6]
+    RandSpace <- c(RandSpace, strsplit(rownames(SpaceEffi), split="Phi for ")[[1]][2])
+  }
+  if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_S", rownames(object$summary.hyperpar))))>0){ # spatial
+    SpaceEffSi <- object$summary.hyperpar[intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_S", rownames(object$summary.hyperpar))), -6]
+    RandSpace <- c(RandSpace, strsplit(rownames(SpaceEffSi), split="Phi for ")[[1]][2])
+  }
   REidentifyL <- REidentify[!REidentify %in% grep("_S",Hnames)] # long
   REidentifyS <- REidentify[REidentify %in% grep("_S",Hnames)] # surv
-  RandEff <- object$summary.hyperpar[REidentifyL,]
-  RandEffS <- object$summary.hyperpar[REidentifyS,]
+  # Exclude the spatial hyperparameters
+  if(!is.null(RandSpace)){
+    spatial_patterns <- c()
+    for(spatial_name in RandSpace) {
+      spatial_patterns <- c(spatial_patterns,
+                            paste0("^Phi for ", spatial_name, "$"),
+                            paste0("^Precision for ", spatial_name, "$"),
+                            paste0("^Range for ", spatial_name, "$"),
+                            paste0("^Stdev for ", spatial_name, "$"),
+                            paste0("^Beta for ", spatial_name, "$"))
+    }
+    spatial_indices <- c()
+    for(pattern in spatial_patterns) {
+      spatial_indices <- c(spatial_indices, grep(pattern, Hnames))
+    }
+    spatial_indices <- unique(spatial_indices)
+
+    REidentifyL <- REidentifyL[which(!(REidentifyL %in% spatial_indices))]
+    REidentifyS <- REidentifyS[which(!(REidentifyS %in% spatial_indices))]
+  }
+  if(length(REidentifyL)>0) RandEff <- object$summary.hyperpar[REidentifyL,] else RandEff <- NULL
+  if(length(REidentifyS)>0) RandEffS <- object$summary.hyperpar[REidentifyS,] else RandEffS <- NULL
   if(!is.null(object$corLong)){
     NRand <- ifelse(object$corLong, 1, length(object$longOutcome))#length(unique(substring(rownames(RandEff), nchar(rownames(RandEff))-1, nchar(rownames(RandEff)))))
   }else{
@@ -186,6 +220,7 @@ summary.INLAjoint <- function(object, ...){
   }
   if(NRand>0){
     NREcur <- 1
+    Refi <- NULL
     ReffList <- vector("list", NRand)
     for(i in 1:NRand){
       if(!is.null(object$fixRE[[i]])){
@@ -197,9 +232,9 @@ summary.INLAjoint <- function(object, ...){
         if(i>9) shiftRE <- 3 else shiftRE <- 2
         RandEffi <- RandEff[which(substring(rownames(RandEff), nchar(rownames(RandEff))-shiftRE, nchar(rownames(RandEff)))==paste0("_L", i)),]
         NRandEffi <- dim(RandEffi)[1]
-        NameRandEffi <- strsplit(rownames(RandEffi)[1], "for ")[[1]][2]
+        if(is.null(NRandEffi)) NRandEffi <- 0
+        if(NRandEffi!=0) NameRandEffi <- strsplit(rownames(RandEffi)[1], "for ")[[1]][2]
         if(NRandEffi==1 | !object$corRE[[i]]){
-          # browser()
           for(j in 1:nrow(RandEffi)){
             NameRandEffi <- strsplit(rownames(RandEffi)[j], "for ")[[1]][2]
             if(!sdcor){
@@ -216,16 +251,66 @@ summary.INLAjoint <- function(object, ...){
               }
             }
             Refi <- cbind("mean" = Varmar$mean,
-                                   "sd" = Varmar$sd,
-                                   "0.025quant" = Varmar$`0.025quant`,
-                                   "0.5quant" = Varmar$`0.5quant`,
-                                   "0.975quant" = Varmar$`0.975quant`)
-            rownames(Refi) <- object$REstruc[NREcur]
+                          "sd" = Varmar$sd,
+                          "0.025quant" = Varmar$`0.025quant`,
+                          "0.5quant" = Varmar$`0.5quant`,
+                          "0.975quant" = Varmar$`0.975quant`)
+            rownames(Refi) <- ifelse(!is.na(object$REstruc[NREcur]), object$REstruc[NREcur], NameRandEffi)
             ReffList[[i]] <- rbind(ReffList[[i]], Refi)
             NREcur <- NREcur + 1
           }
-        }else{
-          NRE = (-1+sqrt(8*NRandEffi+1))/2 # get number of rancom effects from length of Cholesky terms
+        }else if(NRandEffi>1 & length(object$REstruc)==1){
+          for(j in 1:nrow(RandEffi)){
+            NameRandEffi <- strsplit(rownames(RandEffi)[j], "for ")[[1]][2]
+            if(!sdcor){
+              if(TRUE %in% c(c("Inf", "NaN") %in% RandEffi)){ # in case of infinite or not a number in the random effect hyperparameter
+                Varmar <- RandEffi[j,]
+              }else{
+                Varmar <- m.lstat.2(eval(parse(text=paste0("object$internal.marginals.hyperpar$`Log precision for ", NameRandEffi, "`"))))
+              }
+            }else{
+              if(TRUE %in% c(c("Inf", "NaN") %in% RandEffi)){ # in case of infinite or not a number in the random effect hyperparameter
+                Varmar <- RandEffi[j,]
+              }else{
+                Varmar <- m.lstat.1(eval(parse(text=paste0("object$internal.marginals.hyperpar$`Log precision for ", NameRandEffi, "`"))))
+              }
+            }
+            Refi <- cbind("mean" = Varmar$mean,
+                          "sd" = Varmar$sd,
+                          "0.025quant" = Varmar$`0.025quant`,
+                          "0.5quant" = Varmar$`0.5quant`,
+                          "0.975quant" = Varmar$`0.975quant`)
+            rownames(Refi) <- ifelse(!is.na(object$REstruc[NREcur]), object$REstruc[NREcur], NameRandEffi)
+            ReffList[[i]] <- rbind(ReffList[[i]], Refi)
+            NREcur <- NREcur + 1
+          }
+        }else if(NRandEffi>1 & length(object$REstruc)==1){
+          for(j in 1:nrow(RandEffi)){
+            NameRandEffi <- strsplit(rownames(RandEffi)[j], "for ")[[1]][2]
+            if(!sdcor){
+              if(TRUE %in% c(c("Inf", "NaN") %in% RandEffi)){ # in case of infinite or not a number in the random effect hyperparameter
+                Varmar <- RandEffi[j,]
+              }else{
+                Varmar <- m.lstat.2(eval(parse(text=paste0("object$internal.marginals.hyperpar$`Log precision for ", NameRandEffi, "`"))))
+              }
+            }else{
+              if(TRUE %in% c(c("Inf", "NaN") %in% RandEffi)){ # in case of infinite or not a number in the random effect hyperparameter
+                Varmar <- RandEffi[j,]
+              }else{
+                Varmar <- m.lstat.1(eval(parse(text=paste0("object$internal.marginals.hyperpar$`Log precision for ", NameRandEffi, "`"))))
+              }
+            }
+            Refi <- cbind("mean" = Varmar$mean,
+                          "sd" = Varmar$sd,
+                          "0.025quant" = Varmar$`0.025quant`,
+                          "0.5quant" = Varmar$`0.5quant`,
+                          "0.975quant" = Varmar$`0.975quant`)
+            rownames(Refi) <- ifelse(!is.na(object$REstruc[NREcur]), object$REstruc[NREcur], NameRandEffi)
+            ReffList[[i]] <- rbind(ReffList[[i]], Refi)
+            NREcur <- NREcur + 1
+          }
+        }else if(NRandEffi>0){
+          NRE = (-1+sqrt(8*NRandEffi+1))/2 # get number of random effects from length of Cholesky terms
           if(!sdcor){
             MC_samples <- INLA::inla.iidkd.sample(NsampRE, object, NameRandEffi, return.cov=TRUE)
           }else{
@@ -269,7 +354,7 @@ summary.INLAjoint <- function(object, ...){
           ReffList[[i]] <- object$mat_k[[i]]
         }
         # if(NRE>1){
-          rownames(ReffList[[i]]) <- colnames(ReffList[[i]]) <- object$REstruc[NREcur:(NREcur+(NRE-1))]
+        rownames(ReffList[[i]]) <- colnames(ReffList[[i]]) <- object$REstruc[NREcur:(NREcur+(NRE-1))]
         # }else{
         #   names(ReffList[[i]]) <- object$REstruc[NREcur:(NREcur+(NRE-1))]
         # }
@@ -282,6 +367,9 @@ summary.INLAjoint <- function(object, ...){
     Nerr <- 1 #  identify error terms
     out$famLongi <- object$famLongi
     FixedEff <- vector("list", NLongi)
+    if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_L", rownames(object$summary.hyperpar))))>0 |
+       length(intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep("_L", rownames(object$summary.hyperpar))))>0) SpaceEff <- vector("list", NLongi)
+    SpaceEff_asso <- TRUE # to avoid duplicating association when spatial included
     for(i in 1:NLongi){
       if(i>9) shiftFE <- 2 else shiftFE <- 1
       FixedEffi <- object$summary.fixed[which(substring(rownames(object$summary.fixed), nchar(rownames(object$summary.fixed))-shiftFE, nchar(rownames(object$summary.fixed)))==paste0("L", i)), -which(colnames(object$summary.fixed)%in%c("mode","kld"))]
@@ -296,23 +384,85 @@ summary.INLAjoint <- function(object, ...){
       }else if (object$famLongi[i]=="pom"){
         FixedEff[[i]] <- rbind(object$summary.hyperpar[grep("POM", Hnames),-6], FixedEffi)
       }else if(object$famLongi[i]=="nbinomial"){
-        FixedEff[[i]] <- rbind(object$summary.hyperpar[grep("nbinomial", Hnames),-6], FixedEffi)
+        ZIP_p <- object$summary.hyperpar[grep("nbinomial", Hnames),-6]
+        rownames(ZIP_p) <- "zero-infl. probability"
+        FixedEff[[i]] <- rbind(ZIP_p, FixedEffi)
       }else if(object$famLongi[i]=="Betabinomial"){
-        FixedEff[[i]] <- rbind(object$summary.hyperpar[grep("betabinomial", Hnames),-6], FixedEffi)
+        ZIP_p <- object$summary.hyperpar[grep("betabinomial", Hnames),-6]
+        rownames(ZIP_p) <- "zero-infl. probability"
+        FixedEff[[i]] <- rbind(ZIP_p, FixedEffi)
       }else if(object$famLongi[i]=="gpoisson"){
-        FixedEff[[i]] <- rbind(object$summary.hyperpar[grep("gpoisson", Hnames),-6], FixedEffi)
+        ZIP_p <- object$summary.hyperpar[grep("gpoisson", Hnames),-6]
+        rownames(ZIP_p) <- "zero-infl. probability"
+        FixedEff[[i]] <- rbind(ZIP_p, FixedEffi)
       }else if(length(grep("zeroinflated", object$famLongi[i]))>0){
-        FixedEff[[i]] <- rbind(object$summary.hyperpar[grep("zero-inflated", Hnames),-6], FixedEffi)
+        ZIP_p <- object$summary.hyperpar[grep("zero-inflated", Hnames),-6]
+        rownames(ZIP_p) <- "zero-infl. probability"
+        FixedEff[[i]] <- rbind(ZIP_p, FixedEffi)
+      }else if(object$famLongi[i]=="0poisson"){
+        # Handle 0poisson family with zero-inflation logistic parameters
+        ZIP_beta_indices <- grep("beta.*for 0poisson", Hnames)
+        if(length(ZIP_beta_indices) > 0){
+          ZIP_betas <- object$summary.hyperpar[ZIP_beta_indices, -6]
+          # Clean up the names by removing "observations" suffix
+          rownames(ZIP_betas) <- gsub(" observations$", "", rownames(ZIP_betas))
+          FixedEff[[i]] <- rbind(ZIP_betas, FixedEffi)
+        } else {
+          FixedEff[[i]] <- FixedEffi
+        }
       }else{
         FixedEff[[i]] <- FixedEffi
       }
+      if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))))>0){ # spatial
+        SpaceEffi <- object$summary.hyperpar[intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+        SpaceEff[[i]] <- rbind(SpaceEffi,
+                               object$summary.hyperpar[intersect(which(substr(rownames(object$summary.hyperpar), 1, 14)=="Precision for "), which(sapply(strsplit(rownames(object$summary.hyperpar), split="Precision for "), function(x) x[2]==strsplit(rownames(SpaceEffi), split="Phi for ")[[1]][2]))), -6])
+        SpaceEffi2 <- object$summary.hyperpar[intersect(grep("Beta for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+        if(exists("SpaceEffi2")){
+          if(nrow(out$AssocLS)>0){
+            for (j in 1:nrow(out$AssocLS)){ # check if asso is already handled, otherzise just add it here
+              if(identical(c(SpaceEffi2), c(out$AssocLS[j,]))) SpaceEff_asso <- FALSE
+            }
+          }
+          if(nrow(out$AssocSS)>0){
+            for (j in 1:nrow(out$AssocSS)){ # check if asso is already handled, otherzise just add it here
+              if(identical(c(SpaceEffi2), c(out$AssocSS[j,]))) SpaceEff_asso <- FALSE
+            }
+          }
+          if(SpaceEff_asso) SpaceEff[[i]] <- rbind(SpaceEff[[i]], SpaceEffi2)
+        }
+        rm("SpaceEffi2")
+      }
+      if(length(intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))))>0){ # spatial
+        SpaceEffi <- object$summary.hyperpar[intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+        SpaceEff[[i]] <- rbind(SpaceEffi,
+                               object$summary.hyperpar[intersect(which(substr(rownames(object$summary.hyperpar), 1, 10)=="Stdev for "), which(sapply(strsplit(rownames(object$summary.hyperpar), split="Stdev for "), function(x) x[2]==strsplit(rownames(SpaceEffi), split="Range for ")[[1]][2]))), -6])
+        SpaceEffi2 <- object$summary.hyperpar[intersect(grep("Beta for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+        if(exists("SpaceEffi2")){
+          if(nrow(out$AssocLS)>0){
+            for (j in 1:nrow(out$AssocLS)){ # check if asso is already handled, otherzise just add it here
+              if(identical(c(SpaceEffi2), c(out$AssocLS[j,]))) SpaceEff_asso <- FALSE
+            }
+          }
+          if(nrow(out$AssocSS)>0){
+            for (j in 1:nrow(out$AssocSS)){ # check if asso is already handled, otherzise just add it here
+              if(identical(c(SpaceEffi2), c(out$AssocSS[j,]))) SpaceEff_asso <- FALSE
+            }
+          }
+          if(SpaceEff_asso) SpaceEff[[i]] <- rbind(SpaceEff[[i]], SpaceEffi2)
+        }
+        rm("SpaceEffi2")
+      }
     }
     out$FixedEff <- FixedEff
+    if(exists("SpaceEff")) if(!is.null(SpaceEff[[1]])) out$SpaceEff <- SpaceEff
   }
   if(NSurv>0){
     BH <- as.data.frame(BH)
     BHW <- vector("list", NSurv) # baseline risk
     SurvEff <- vector("list", NSurv)
+    if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep("_S", rownames(object$summary.hyperpar))))>0) SpaceEffS <- vector("list", NSurv)
+    SpaceEffS_asso <- TRUE
     BaselineValues <- vector("list", NSurv) # values of piecewise constant baseline
     nbl <- 1 # to keep track of baseline risk in case of multiple survival outcomes
     nbl2 <- 1 # to keep track of baseline risk in case of multiple survival outcomes
@@ -325,6 +475,16 @@ summary.INLAjoint <- function(object, ...){
         BHW[[i]] <- object$summary.hyperpar[grep("weibull", Hnames),][nbl2,-6]
         rownames(BHW[[i]]) <- paste0("Weibull (shape)_S", i)
         nbl2 <- nbl2+1
+      }else if(object$basRisk[[i]]=="dgompertzsurv"){
+        nameRisk <- "Defective Gompertz (scale)"
+        BHW[[i]] <- object$summary.hyperpar[grep("dGompertz", Hnames),][nbl2,-6]
+        rownames(BHW[[i]]) <- paste0("Defective Gompertz (alpha)_S", i)
+        nbl2 <- nbl2+1
+      }else if(object$basRisk[[i]]=="gompertzsurv"){
+        nameRisk <- "Gompertz (scale)"
+        BHW[[i]] <- object$summary.hyperpar[grep("Gompertz", Hnames),][nbl2,-6]
+        rownames(BHW[[i]]) <- paste0("Gompertz (alpha)_S", i)
+        nbl2 <- nbl2+1
       }else{
         nameRisk <- "Baseline risk (mean)"
         BHW[[i]] <- BH[nbl,]
@@ -357,11 +517,21 @@ summary.INLAjoint <- function(object, ...){
                                      median=BHme,
                                      upper=BHup)
       }
-      if(object$basRisk[[i]] %in% c("exponentialsurv", "weibullsurv") & !is.null(object$cureVar[[i]])){
-        MCure <- object$summary.hyperpar[grep("Weibull-Cure", Hnames),-6]
-        rownames(MCure) <- object$cureVar[[i]]
+      if(object$basRisk[[i]] %in% c("exponentialsurv", "weibullsurv", "dgompertzsurv", "gompertzsurv") & !is.null(object$cureVar[[i]])){
+        # Look for various cure model patterns
+        cure_patterns <- c("Weibull-Cure", "Exponential-Cure", "Dgompertz-Cure", "Gompertz-Cure")
+        cure_idx <- NULL
+        for(pattern in cure_patterns){
+          cure_idx <- grep(pattern, Hnames, ignore.case=TRUE)
+          if(length(cure_idx) > 0) break
+        }
+        if(length(cure_idx) > 0){
+          MCure <- object$summary.hyperpar[cure_idx,-6]
+          rownames(MCure) <- object$cureVar[[i]]
+        }
       }
-      SurvEffi <- rbind(BHW[[i]], object$summary.fixed[which(substring(rownames(object$summary.fixed), nchar(rownames(object$summary.fixed))-1, nchar(rownames(object$summary.fixed)))==paste0("S", i)), -which(colnames(object$summary.fixed)%in%c("mode","kld"))])
+      if(i<10) NCR = 1 else NCR = 2
+      SurvEffi <- rbind(BHW[[i]], object$summary.fixed[which(substring(rownames(object$summary.fixed), nchar(rownames(object$summary.fixed))-NCR, nchar(rownames(object$summary.fixed)))==paste0("S", i)), -which(colnames(object$summary.fixed)%in%c("mode","kld"))])
       rownames(SurvEffi) <- gsub("\\.X\\.", ":", rownames(SurvEffi))
       rownames(SurvEffi)[grep("Intercept", rownames(SurvEffi))] <- paste0(nameRisk, "_S", i)
       if(!is.null(object$marginals.fixed[[paste0("Intercept_S",i)]])){
@@ -380,6 +550,8 @@ summary.INLAjoint <- function(object, ...){
           #inla.posterior.sample.eval(function(...){Intercept_S1*alpha parameter for weibullsurv} , SMPLweib)
           # need to do the alpha*Beta0 for intercept ("scale" (should it be renamed?)) here and exp(alpha*betas) below for hazard ratios
           #}
+        }else if(object$variant==10 | object$variant==01){ # transform from variant 1 to variant 0
+          stop("Implementation of variant 01/10 under work")
         }
 
         SurvEffi[paste0(nameRisk, "_S", i), "mean"] <- trsf$mean
@@ -393,7 +565,8 @@ summary.INLAjoint <- function(object, ...){
           if(!j%in%c(grep("aseline", rownames(SurvEffi)),
                      grep("(rate)", rownames(SurvEffi)),
                      grep("(shape)", rownames(SurvEffi)),
-                     grep("(scale)", rownames(SurvEffi)))){
+                     grep("(scale)", rownames(SurvEffi)),
+                     grep("(alpha)", rownames(SurvEffi)))){
             RNM <- gsub(":", "\\.X\\.", rownames(SurvEffi)[j])
             m <- INLA::inla.smarginal(object$marginals.fixed[[RNM]])
             ab <- INLA::inla.qmarginal(c(0.001, 0.999), m)
@@ -410,6 +583,69 @@ summary.INLAjoint <- function(object, ...){
           }
         }
       }
+      if(length(intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))))>0){ # spatial
+        if(exists("SpaceEff")){
+          if(rownames(object$summary.hyperpar[intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))),]) %in% c(sapply(SpaceEff, rownames))){
+            SpaceEllDONE <- FALSE
+          }else{
+            SpaceEllDONE <- TRUE
+          }
+        }else{
+          SpaceEllDONE <- TRUE
+        }
+        if(SpaceEllDONE){
+          SpaceEffSi <- object$summary.hyperpar[intersect(grep("Phi for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))), -6]
+          SpaceEffS[[i]] <- rbind(SpaceEffSi,
+                                  object$summary.hyperpar[intersect(which(substr(rownames(object$summary.hyperpar), 1, 14)=="Precision for "), which(sapply(strsplit(rownames(object$summary.hyperpar), split="Precision for "), function(x) x[2]==strsplit(rownames(SpaceEffSi), split="Phi for ")[[1]][2]))), -6])
+          SpaceEffSi2 <- object$summary.hyperpar[intersect(grep("Beta for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))), -6]
+          if(dim(SpaceEffSi2)[1]==0) SpaceEffSi2 <- object$summary.hyperpar[intersect(grep("Beta for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+          if(exists("SpaceEffSi2")){
+            if(nrow(out$AssocLS)>0){
+              for (j in 1:nrow(out$AssocLS)){ # check if asso is already handled, otherzise just add it here
+                if(!identical(c(SpaceEffSi2), c(out$AssocLS[j,]))) SpaceEffS_asso <- FALSE
+              }
+            }
+            if(nrow(out$AssocSS)>0){
+              for (j in 1:nrow(out$AssocSS)){ # check if asso is already handled, otherzise just add it here
+                if(!identical(c(SpaceEffSi2), c(out$AssocSS[j,]))) SpaceEffS_asso <- FALSE
+              }
+            }
+            if(SpaceEffS_asso) SpaceEffS[[i]] <- rbind(SpaceEffS[[i]], SpaceEffSi2)
+          }
+          rm("SpaceEffSi2")
+        }
+      }
+      if(length(intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))))>0){ # spatial
+        if(exists("SpaceEff")){
+          if(rownames(object$summary.hyperpar[intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep(paste0("_S", i), rownames(object$summary.hyperpar))),]) %in% c(sapply(SpaceEff, rownames))){
+            SpaceEllDONE <- FALSE
+          }else{
+            SpaceEllDONE <- TRUE
+          }
+        }else{
+          SpaceEllDONE <- TRUE
+        }
+        if(SpaceEllDONE){
+          SpaceEffSi <- object$summary.hyperpar[intersect(grep("Range for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+          SpaceEffS[[i]] <- rbind(SpaceEffSi,
+                                  object$summary.hyperpar[intersect(which(substr(rownames(object$summary.hyperpar), 1, 10)=="Stdev for "), which(sapply(strsplit(rownames(object$summary.hyperpar), split="Stdev for "), function(x) x[2]==strsplit(rownames(SpaceEffSi), split="Range for ")[[1]][2]))), -6])
+          SpaceEffSi2 <- object$summary.hyperpar[intersect(grep("Beta for ", rownames(object$summary.hyperpar)), grep(paste0("_L", i), rownames(object$summary.hyperpar))), -6]
+          if(exists("SpaceEffSi2")){
+            if(nrow(out$AssocLS)>0){
+              for (j in 1:nrow(out$AssocLS)){ # check if asso is already handled, otherzise just add it here
+                if(!identical(c(SpaceEffSi2), c(out$AssocLS[j,]))) SpaceEffS_asso <- FALSE
+              }
+            }
+            if(nrow(out$AssocSS)>0){
+              for (j in 1:nrow(out$AssocSS)){ # check if asso is already handled, otherzise just add it here
+                if(!identical(c(SpaceEffSi2), c(out$AssocSS[j,]))) SpaceEffS_asso <- FALSE
+              }
+            }
+            if(SpaceEffS_asso) SpaceEffS[[i]] <- rbind(SpaceEffS[[i]], SpaceEffSi2)
+          }
+          rm("SpaceEffSi2")
+        }
+      }
       if(!is.null(MCure)){
         if(hr) rownames(MCure) <- paste0(rownames(MCure), " (not exp!)")
         SurvEffi <- rbind(MCure, SurvEffi)
@@ -420,11 +656,16 @@ summary.INLAjoint <- function(object, ...){
       SurvEff[[i]] <- SurvEffi
     }
     out$SurvEff <- SurvEff
+    if(exists("SpaceEffS")) if(!is.null(SpaceEffS[[1]])) out$SpaceEffS <- SpaceEffS
     if(NRandS>0){
       NREcurS <- 1
       ReffListS <- vector("list", NSurv)
       for(i in 1:NRandS){
         RandEffiS <- RandEffS[which(substring(rownames(RandEffS), nchar(rownames(RandEffS)), nchar(rownames(RandEffS)))==i),]
+        if(dim(RandEffiS)[1]==0){ # only one random effect not in first submodel
+          RandEffiS <- RandEffS
+          i=as.integer(substring(rownames(RandEffS), nchar(rownames(RandEffS)), nchar(rownames(RandEffS))))
+        }
         NRandEffiS <- dim(RandEffiS)[1]
         NameRandEffiS <- strsplit(rownames(RandEffiS)[1], "for ")[[1]][2]
         if(NRandEffiS==1){
@@ -466,6 +707,7 @@ summary.INLAjoint <- function(object, ...){
   out$NRandS <- NRandS
   out$mlik <- object$mlik
   out$cpu.used <- object$cpu.used
+  out$rw2_info <- object$rw2_info
   class(out) <- "summary.INLAjoint"
   out
 }
